# Python's Libraries
import logging

# Third-party Libraries
from unittest import TestCase

# Own's Libraries
from stx_logger_util.log_admin import LogAdmin
from stx_logger_util.decorators import log
from stx_logger_util.log_admin import LogEnv


@log()
def example_Function(_parameter):
    logging.critical("Executing the example function on Production ... critical")
    logging.error("Executing the example function on Production ... error")
    logging.warning("Executing the example function on Production ... warning")
    logging.info("Executing the example function on Production ... info")
    logging.debug("Executing the example function on Production ... debug")


@log(_level=2)
def example_FunctionWithLevel(_parameter, _other_parameter=None):
    logging.critical("Executing the example function with level ... critical")
    logging.error("Executing the example function with level ... error")
    logging.warning("Executing the example function with level ... warning")
    logging.info("Executing the example function with level ... info")
    logging.debug("Executing the example function with level ... debug")


@log(_level=2.2)
def example_FunctionWithSublevel(_parameter, _other_parameter=None):
    logging.critical("Executing the example function with sublevel ... critical")
    logging.error("Executing the example function with sublevel ... error")
    logging.warning("Executing the example function with sublevel ... warning")
    logging.info("Executing the example function with sublevel ... info")
    logging.debug("Executing the example function with sublevel ... debug")


@log(_level=3)
def example_FunctionWithError(_parameter):
    raise NameError("Example Error")


class ExampleObject:

    @log()
    def get_TestData(self, _param1, _param2):
        return "ok"


class StepFunctionTest(TestCase):

    def test_Given_RequiredParams_When_EnvironmentIsProduction_Then_DontShowLogs(self):
        LogAdmin.create_Logger(LogEnv.PRODUCTION, "test1")
        example_Function("cadenita")

    def test_Given_RequiredAndTimestampParams_When_EnvironmentIsProductionAndTimestampIsFalse_Then_DontShowLogs(self):
        LogAdmin.create_Logger(LogEnv.PRODUCTION, "test2", False)
        example_Function(12)

    def test_Given_RequiredAndTimestampParams_When_EnvironmentIsProductionAndTimestampIsTrue_Then_DontShowLogs(self):
        LogAdmin.create_Logger(LogEnv.PRODUCTION, "test2", True)
        example_Function(12)

    def test_Given_RequiredParams_When_EnvironmentIsDevelopment_Then_ShowLogsWithoutTimestamp(self):
        LogAdmin.create_Logger(LogEnv.DEVELOPMENT, "test1")
        example_Function("cadenita")

    def test_Given_RequiredAndTimestapParams_When_EnvironmentIsDevelopmentAndTimestampIsFalse_Then_ShowLogsWithoutTimestamp(self):
        LogAdmin.create_Logger(LogEnv.DEVELOPMENT, "test2", False)
        example_Function(23)

    def test_Given_RequiredAndTimestapParams_When_EnvironmentIsDevelopmentAndTimestampIsTrue_Then_ShowLogsWithTimestamp(self):
        LogAdmin.create_Logger(LogEnv.DEVELOPMENT, "test2", True)
        example_Function(23)

    def test_Given_RequiredParams_When_EnvironmentIsDevelopAndFunctionHasLevel_Then_ShowLogsWithoutTimestampAndLevelDots(self):
        LogAdmin.create_Logger(LogEnv.DEVELOPMENT, "test1")
        example_FunctionWithLevel("cadenita")

    def test_Given_RequiredParams_When_EnvironmentIsDevelopAndFunctionHasSublevel_Then_ShowLogsWithoutTimestampAndSublevelDots(self):
        LogAdmin.create_Logger(LogEnv.DEVELOPMENT, "test1")
        example_FunctionWithSublevel("cadenita", "martillo")

    def test_Given_RequiredParams_When_EnvironmentIsProductionAndFunctionHasSublevelAndFail_Then_ShowLogsWithoutTimestampAndSublevelDots(self):
        LogAdmin.create_Logger(LogEnv.PRODUCTION, "test1")
        with self.assertRaises(NameError):
            example_FunctionWithError("cadenita")


class StepClassTest(TestCase):

    def test_Given_RequiredParams_When_EnvironmentIsDevelopment_Then_ShowLogs(self):
        LogAdmin.create_Logger(LogEnv.DEVELOPMENT, "test")
        example_obj = ExampleObject()
        var1 = "Parametro 1"
        var2 = "Parametro 2"
        example_obj.get_TestData(var1, var2)



# Own's Libraries
from .function_types import FunctionType


class LogMessage(object):

    @classmethod
    def __clean_ParamValue(self, _param):
        value = _param

        if _param is None:
            value = "None"

        return value

    @classmethod
    def __build_ParamString(self, _params_str, _qty_flag, _qty_args, _value):
        if _qty_flag == _qty_args:
            _params_str += f"{_value}"
        else:
            _params_str += f"{_value}, "

        return _params_str

    @classmethod
    def __build_Prefix(self, _level):
        prefix = ""
        for n in range(int(_level)):
            if n == 0:
                continue

            prefix += "..."

        if isinstance(_level, float):
            level_str = str(_level)
            level_part = level_str.split('.')[1]

            if int(level_part) > 2:
                raise ValueError("Decimal part is greates than 2")

            for n in range(int(level_part)):
                prefix += "."

        return prefix

    @classmethod
    def __get_Message(self, _pre_message, _level):
        prefix = self.__build_Prefix(_level)

        if prefix:
            message = f"{prefix} {_pre_message}"
        else:
            message = _pre_message

        return message

    @classmethod
    def get_Start(self, _level, _args, _func_path, _func_type):
        qty_flag = 0
        qty_args = len(_args)
        params_str = ""

        for param in _args:
            qty_flag += 1

            if _func_type == FunctionType.METHOD \
                    and qty_flag == 1:
                continue

            value = self.__clean_ParamValue(param)
            params_str = self.__build_ParamString(params_str, qty_flag, qty_args, value)

        pre_message = f"{_func_path}({params_str})"
        message = self.__get_Message(pre_message, _level)

        return message

    @classmethod
    def get_End(self, _level, _result, _func_path):
        pre_message = f"{_func_path}: {_result}"
        return self.__get_Message(pre_message, _level)



# Python's Libraries
import logging
from enum import Enum


loggers = {}


class LogEnv(Enum):
    PRODUCTION = "PRODUCTION"
    DEVELOPMENT = "DEVELOPMENT"
    TEST = "TEST"

    @staticmethod
    def list():
        return list(map(lambda item: item.value, LogEnv))


class LogAdmin(object):

    @classmethod
    def __get_Level(self, _environment):
        if _environment == LogEnv.PRODUCTION:
            return logging.INFO

        return logging.DEBUG

    @classmethod
    def __get_MsgFormat(self, _timestamp):
        if _timestamp:
            return "%(asctime)s [%(levelname)s] %(message)s"

        return "[%(levelname)s] %(message)s"

    @classmethod
    def create_Logger(
        self,
        _environment=LogEnv.DEVELOPMENT,
        _name="app",
        _timestamp=False
    ):
        global loggers

        if loggers.get(_name):
            return loggers.get(_name)

        level = self.__get_Level(_environment)
        msg_format = self.__get_MsgFormat(_timestamp)

        logging.root.handlers = []
        logging.basicConfig(
            format=msg_format,
            datefmt='%m/%d/%Y %I:%M:%S %p' if _timestamp else None,
            level=level
        )

        logger = logging.getLogger(_name)
        loggers[_name] = logger

        return logger


# Only will show the messages of the set level or supperior

# LEVELS:
# - CRITICAL	50
# - ERROR	40
# - WARNING	30
# - INFO	20
# - DEBUG	10
# - NOTSET	0



# Python's Libraries
import logging
import inspect

# Own's Libraries
from .log_message import LogMessage
from .function_types import FunctionType


def get_FunctionType(_func):
    if inspect.getargspec(_func)[0][0] == 'self':
        return FunctionType.METHOD
    else:
        return FunctionType.SIMPLE_FUNCTION


def get_FunctionPath(_func, _args, _function_type):
    if _function_type == FunctionType.METHOD:
        return f'{_func.__module__}.{_args[0].__class__.__name__}.{_func.__name__}'
    else:
        return f"{_func.__module__}.{_func.__name__}"


def log(_level=1):
    def inner(func):
        def wrapper(*args, **kwargs):
            function_type = get_FunctionType(func)
            function_path = get_FunctionPath(func, args, function_type)
            logging.debug(LogMessage.get_Start(_level, args, function_path, function_type))
            try:
                response = func(*args, **kwargs)
                logging.debug(LogMessage.get_End(_level, "Success", function_path))
                return response

            except Exception as e:
                logging.error(LogMessage.get_End(_level, str(e), function_path))
                raise e

        return wrapper
    return inner



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="stx-logger-util",
    packages=setuptools.find_packages(),
    version="0.0.5",
    description="Library for a fast logger implementation.",
    author="Carlos A. Martínez Jiménez",
    author_email="carloxdev@gmail.com",
    url="https://github.com/carloxdev/logger-util",
    keywords=["logger", "logging", "python"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent"
    ],
)
